package com.muyu.common.caffeine;


import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import com.muyu.common.caffeine.constents.CaffeineContent;
import com.muyu.common.redis.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.cache.caffeine.CaffeineCache;
import org.springframework.cache.support.SimpleCacheManager;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Map;

/**
 * @Author: 胡杨
 * @Name: CaffeineUtils
 * @Description: 缓存工具类
 * @CreatedDate: 2024/9/26 下午2:53
 * @FilePath: com.muyu.common.caffeine
 */
@Slf4j
@Component
public class CaffeineCacheUtils {
    @Resource
    private RedisService redisService;
    @Resource
    private SimpleCacheManager simpleCacheManager;


    /**
     * 车辆上线 - 新增缓存
     */
    public void addCarCache(String vin) {
        ArrayList<CaffeineCache> caches = new ArrayList<>();
        // 从Redis中获取缓存信息
        Map<String,Object> cacheMap = redisService.getCacheMap(CaffeineContent.CAR_VIN_KEY +vin);
        cacheMap.forEach((key, value) -> {
            Cache<Object , Object> cache = Caffeine.newBuilder().build();
            cache.put(key, value);
            // 全部存储到 CaffeineCache集合
            caches.add(new CaffeineCache(vin, cache));
        });
        simpleCacheManager.setCaches(caches);
        log.info("车辆编码:{},本地缓存完成...",vin);
    }

    /**
     * 车辆下线 - 删除缓存
     */
    public void deleteCarCache(String vin) {
        if (hasCarVinCache(vin)) {
            log.warn("车辆编码:{},本地缓存不存在该车辆信息...", vin);
            return;
        }
        simpleCacheManager.getCache(vin).invalidate();
        log.info("车辆编码:{},本地缓存删除完成...", vin);
    }

    /**
     * 获取车辆信息缓存
     */
    public Object getCarCache(String vin, String key) {
        if (hasCarVinKeyCache(vin, key)){
            log.warn("车辆编码:{},本地缓存不存在该车辆信息...",vin);
            return null;
        }
        return simpleCacheManager.getCache(vin).get(key).get();
    }

    /**
     * 获取车辆信息缓存
     */
    public <T> T getCarCache(String vin, String key, Class<T> type) {
        if (hasCarVinKeyCache(vin,key)){
            log.warn("车辆编码:{},本地缓存不存在该车辆信息...",vin);
            return null;
        }
        return simpleCacheManager.getCache(vin).get(key, type);
    }

    /**
     * 判断缓存存在与否
     */
    public Boolean hasCarVinCache(String vin) {
        return ObjectUtils.isNotEmpty(simpleCacheManager.getCache(vin));
    }

    /**
     * 判断缓存的Key存在与否
     */
    public Boolean hasCarVinKeyCache(String vin,String key) {
        return hasCarVinCache(vin) && ObjectUtils.isNotEmpty(simpleCacheManager.getCache(vin).get(key).get());
    }
}
