package com.muyu.data.processing.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.data.processing.domain.IotDbData;
import com.muyu.data.processing.service.DataProcessingService;

import javax.annotation.Resource;

import org.springframework.web.bind.annotation.*;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.List;

/**
 * 数据处理控制层
 *
 * @Author: 胡杨
 * @Name: DataProcessing
 * @Description: 数据处理控制层
 * @CreatedDate: 2024/9/28 下午3:53
 * @FilePath: com.muyu.data.processing.controller
 */

@Slf4j
@RestController
@RequestMapping("/DataProcessing")
public class DataProcessingController {
    @Resource
    private DataProcessingService service;



    @RequestMapping(value = "/createCarData", method = RequestMethod.POST)
//    @Transactional(rollbackFor = Exception.class)
    public Result createCarData(@RequestBody IotDbData data) {
        try {
            data.setTimestamp(System.currentTimeMillis());
            data.setCreateTime(new Date());
            Integer v = service.createCarData(data);
            if (v == -1) {
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("创建车辆报文记录失败！" + e);
            return Result.error(false);
        }
    }


    /**
     * 更新操作 其实也是插入操作 时间戳相同 只和时间戳相关
     * @param data
     * @return
     */
    @RequestMapping(value = "/updateCarData", method = RequestMethod.POST)
    public Result updateCarData(@RequestBody IotDbData data) {
        try {
            data.setTimestamp(System.currentTimeMillis());
            data.setCreateTime(new Date());
            Integer v = service.updateCarData(data);
            if (v == -1) {
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("更新车辆报文记录失败！" + e);
            return Result.error(false);
        }
    }


    /**
     * 删除操作  要将时间戳的加号变成%2B
     * @param timestamp
     * @return
     */
    @RequestMapping(value = "/deleteCarData", method = RequestMethod.GET)
    public Result deleteCarData(String timestamp) {
        try {
            Integer v = service.deleteCarData(timestamp);
            if (v == -1) {
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("删除车辆报文记录失败！" + e);
            return Result.error(false);
        }
    }


    /**
     * 创建组 也就是相当于数据库
     * @return
     */
    @RequestMapping(value = "/createCarDataGroup", method = RequestMethod.POST)
    public Result createCarDataGroup() {
        try {
            Integer v = service.createCarDataGroup();
            if (v > 0) {
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("创建车辆报文记录组失败！" + e);
            return Result.error(false);
        }
    }

    /**
     * 查询所有车辆报文记录数据
     * @return
     */
    @RequestMapping(value = "/queryCarData", method = RequestMethod.GET)
    public Result queryCarData() {
        try {
            List<IotDbData> v = service.queryCarData();
            if (v.size() > 0) {
                v.forEach(x -> {
                    System.out.println(x.toString());
                });
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("查询车辆报文记录组失败！" + e);
            return Result.error(false);
        }
    }

    /**
     * 查看数据库有多少组
     *
     * @return
     */
    @RequestMapping(value = "/selectStorageGroup", method = RequestMethod.GET)
    public Result selectStorageGroup() {
        try {
            List<String> v = service.selectStorageGroup();
            if (v.size() > 0) {
                v.forEach(x -> {
                    System.out.println("group------------------" + x.toString());
                });
                return Result.success(v);
            } else {
                return Result.error(false);
            }
        } catch (Exception e) {
            log.info("查询组失败！" + e);
            return Result.error(false);
        }
    }
}
