package com.muyu.data.processing.domain;

import com.muyu.data.processing.domain.req.TestReq;
import com.muyu.data.processing.domain.resp.TestResp;
import com.muyu.data.processing.strategy.StrategyHandler;
import com.muyu.data.processing.strategy.branch.OneBranchStrategy;
import com.muyu.data.processing.strategy.branch.TwoBranchStrategy;
import com.muyu.data.processing.strategy.leaves.FourLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.OneLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.ThreeLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.TwoLeavesStrategy;
import lombok.Getter;

import java.util.Arrays;

/**
 * 策略选择枚举
 *
 * @Author: 胡杨
 * @Name: StrategyEums
 * @Description: 策略选择枚举
 * @CreatedDate: 2024/9/28 上午11:59
 * @FilePath: com.muyu.data.processing.domain
 */

@Getter
public enum StrategyEums {
    TEST1("加减法", new OneBranchStrategy()),
    TEST2("乘除法", new TwoBranchStrategy()),
    TEST1_1("加法", new OneLeavesStrategy()),
    TEST1_2("减法", new TwoLeavesStrategy()),
    TEST2_1("乘法", new ThreeLeavesStrategy()),
    TEST2_2("除法", new FourLeavesStrategy());

    private final String code;
    private final StrategyHandler<TestReq, TestResp> info;

    StrategyEums(String code, StrategyHandler<TestReq, TestResp> info) {
        this.code = code;
        this.info = info;
    }

    /**
     * 鉴别参数是否是枚举的值
     *
     * @param code 需鉴别参数
     * @return 如果存在返回结果turn, 否则返回false
     */
    public static boolean isCode(String code) {
        return Arrays.stream(values())
                .map(StrategyEums::getCode)
                .anyMatch(c -> c.equals(code));
    }

    public static StrategyHandler<TestReq, TestResp> getStrategy(String code) {
        return Arrays.stream(values())
                .filter(c -> c.getCode().equals(code))
                .findFirst()
                .orElseThrow(() -> new IllegalArgumentException("参数错误"))
                .getInfo();
    }
}
