package com.muyu.data.processing.rebbit;


import com.muyu.common.caffeine.CaffeineCacheUtils;
import com.rabbitmq.client.Channel;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashSet;

/**
 * @Author: 胡杨
 * @Name: DownlineRabbit
 * @Description: 车辆下线监听器
 * @CreatedDate: 2024/9/26 下午8:21
 * @FilePath: com.muyu.data.processing.rebbit
 */
@Slf4j
@Component
public class DownlineRabbit {
    private CaffeineCacheUtils caffeineCacheUtils = new CaffeineCacheUtils();

    private static final HashSet<String> DOWNLINE_SET = new HashSet<>();

    @RabbitListener(queuesToDeclare = {@Queue("Downline")})
    public void downline(String vin, Message message, Channel channel) {
        log.info("车辆 {} 下线, 配置信息准备中。。。",vin);
        try {
            // 重复性校验
            if (DOWNLINE_SET.add(message.getMessageProperties().getMessageId())) {
                caffeineCacheUtils.deleteCarCache(vin);
                log.info("车辆 {} 下线, 消息已确认。。。",vin);
            } else {
                log.info("车辆 {} 下线, 消息重复消费，已确认。。。",vin);
            }
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            log.info("车辆 {} 下线, 配置信息已准备完毕。。。",vin);
        } catch (IOException e) {
            try {
                log.warn("车辆 {} 下线, 配置信息准备失败，返回队列，原因：{}", vin, e.getMessage());
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);
            } catch (IOException ex) {
                log.warn("车辆 {} 下线, 消息返回队列失败，原因：{}", vin, ex.getMessage());
            }
        }
    }
}
