package com.muyu.data.processing.strategy.branch;

import com.muyu.data.processing.domain.StrategyEums;
import com.muyu.data.processing.domain.req.TestReq;
import com.muyu.data.processing.domain.resp.TestResp;
import com.muyu.data.processing.strategy.StrategyHandler;
import com.muyu.data.processing.strategy.abstractStrategyRouter;
import com.muyu.data.processing.strategy.leaves.FourLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.OneLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.ThreeLeavesStrategy;
import com.muyu.data.processing.strategy.leaves.TwoLeavesStrategy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * 1号分支策略方法实现
 *
 * @Author: 胡杨
 * @Name: OneBranchStrategy
 * @Description: 1号叶子策略方法实现
 * @CreatedDate: 2024/9/28 上午11:50
 * @FilePath: com.muyu.data.processing.strategy.impl
 */

@Slf4j
@Component
public class OneBranchStrategy extends abstractStrategyRouter<TestReq, TestResp> implements StrategyHandler<TestReq,TestResp> {
    @Override
    public TestResp apply(TestReq testReq) {
        log.info("1号分支策略方法实现,参数1:{},参数2:{},执行方法:{}", testReq.getOne(), testReq.getTwo(), testReq.getType2());
        return applyStrategy(testReq);
    }

    @Override
    protected StrategyMapper<TestReq, TestResp> registerStrategy() {
        return param -> StrategyEums.getStrategy(param.getType2());
    }
}
